@@ -119,11 +119,14 @@ SEQUENCE_TYPES = {
 
 NestedDict = Dict[_KT,Dict]
 RenameMap = Dict[str,str]
+TypeMap = Dict[str,Type]
 
 NestedData = Union[NestedSequence, NestedDict]
 TabularData = Union[Records, DataFrame]
 MappingData = Union[Records, DataFrame, Dict]
+
 Data = Union[Records, DataFrame, Dict, List, NestedSequence, NestedDict]
+MappedData = Dict[str,Data]
 
 JsonData = Union[Dict, List]
 JSON_DATA = (Dict, List)
@@ -143,7 +146,7 @@ PostData = Union[Dict[str,Any],str]
 ResponseData = Union[Records, DataFrame, Dict, List, NestedSequence, NestedDict, str, Tag, List[str], List[Tag]]
 
 DATA_TYPES = {
-    "NestedDict":NestedDict, "RenameMap":RenameMap, "NestedDict":NestedDict,
+    "NestedDict":NestedDict, "RenameMap":RenameMap, "TypeMap":TypeMap, "NestedData":NestedData,
     "TabularData":TabularData, "MappingData":MappingData, "Data":Data, "JsonData":JsonData,
     "RedirectData":RedirectData, "HtmlData":HtmlData, "Account":Account, "PostData":PostData,
     "PandasData":PandasData, "ResponseData":ResponseData}
@@ -211,25 +214,46 @@ TYPE_LIST = {
     Tag: TAG_TYPES,
 }
 
-abs_idx = lambda idx: abs(idx+1) if idx < 0 else idx
+_abs_idx = lambda idx: abs(idx+1) if idx < 0 else idx
+_arg_get = lambda arr, idx: arr[idx if _abs_idx(idx) < len(arr) else -1] if arr else None
 
 
-def get_type(__object: Union[Type,TypeHint,Any], argidx=0) -> Type:
+def get_type(__object: Union[Type,TypeHint,Any], argidx=0) -> Union[Type,Tuple[Type]]:
     if isinstance(__object, Type): return __object
-    elif isinstance(__object, str):
-        types = [__t for __t, __hint in TYPE_LIST.items() if __object.lower() in __hint]
-        if types: return types[0]
-        else: raise ValueError(INVALID_TYPE_HINT_MSG(__object))
+    elif isinstance(__object, str): return _get_type_from_str(__object, argidx)
+    else: return _get_type_from_origin(__object, argidx)
+
+
+def _get_type_from_str(__object: str, argidx=0) -> Union[Type,Tuple[Type]]:
+    if __object.startswith("Union[") and __object.endswith("]"):
+        types = [get_type(__t) for __t in re.search(r"^Union\[([^]]+)\]$", __object).groups()[0].split(',')]
+        return _arg_get(types, argidx)
+    elif __object.startswith("Tuple[") and __object.endswith("]"):
+        return tuple(get_type(__t) for __t in re.search(r"^Tuple\[([^]]+)\]$", __object).groups()[0].split(','))
+    for __t, __hint in TYPE_LIST.items():
+        if __object.lower() in __hint: return __t
+
+
+def _get_type_from_origin(__object: Union[Type,TypeHint,Any], argidx=0) -> Union[Type,Tuple[Type]]:
     __type, args = get_origin(__object), get_args(__object)
-    if args:
-        return get_type(args[argidx if abs_idx(argidx) < len(args) else -1])
+    if __type in (Tuple, tuple):
+        return tuple(get_type(arg) for arg in args)
+    elif args:
+        if __type == Union: return get_type(_arg_get(args, argidx))
+        else: return get_type(__type)
     elif isinstance(__object, Callable):
         return get_type(__type if __type else get_type_hints(__object).get("return"))
     else: return type(__object)
 
 
-def init_origin(__object: Union[Type,TypeHint,Any], argidx=0, default=None) -> Any:
+def init_origin(__object: Union[Type,TypeHint,Any], default=None, argidx=0) -> Any:
     __type = get_type(__object, argidx)
+    if isinstance(__type, Tuple):
+        return tuple(init_type(__t, default) for __t in __type)
+    else: return init_type(__type, default)
+
+
+def init_type(__type: Type, default=None) -> Any:
     try: return __type()
     except TypeError:
         if __type == datetime: return default if default else datetime.now()
@@ -449,32 +473,6 @@ def is_iterable_annotation(annotation, custom_type=True) -> bool:
     return any(map(lambda x: x in name, iterable))
 
 
-def inspect_source(func: Callable) -> Dict[_KT,TypeHint]:
-    info, source = dict(), inspect.getsource(func)
-    return_annotation = inspect.signature(func).return_annotation
-    returns = re.search(r"-> ([^:]+):", source).groups()[0] if return_annotation != inspect._empty else None
-    pattern = re.compile(f"def {func.__name__}\(((.|[^:]\n)*)\)"+(r" ->" if returns else r":\n"))
-    catch = pattern.search(source)
-    if catch:
-        raw = catch.groups(0)[0]
-        params = map(lambda x: x.split(": "), raw.replace('\n',' ').split(", "))
-        info = {param[0].strip(): _decap(param[1].split('=')[0]) for param in params if len(param) == 2}
-    return dict(info, **({"__return__":returns} if returns else dict()))
-
-
-def inspect_function(func: Callable, ignore: Sequence[_KT]=list()) -> Dict[_KT,Dict]:
-    params = dict()
-    signature = inspect.signature(func)
-    type_hint = inspect_source(func)
-    for name, parameter in signature.parameters.items():
-        if name in ignore: continue
-        else: params[name] = inspect_parameter(parameter, type_hint.get(name))
-    return_annotation = inspect.signature(func).return_annotation
-    if return_annotation != inspect._empty:
-        params["__return__"] = inspect_annotation(return_annotation, type_hint.get("__return__"))
-    return params
-
-
 def inspect_parameter(parameter: Parameter, __type: Optional[TypeHint]=None) -> Dict:
     annotation = __type if __type else parameter.annotation
     info = dict(annotation=get_annotation_name(annotation), type=get_annotation_type(annotation))
