@@ -3,8 +3,8 @@ from gscraper.base.types import Index, IndexLabel, Keyword, Unit, IndexedSequenc
 from gscraper.base.types import Records, MappingData, TabularData, Data, PandasData, HtmlData, ResponseData
 from gscraper.base.types import ApplyFunction, MatchFunction, BetweenRange, RegexFormat, PANDAS_DATA
 from gscraper.base.types import is_bool_array, is_int_array, is_array, is_2darray, is_records, is_dfarray, is_tag_array
-from gscraper.base.types import is_comparable, is_list_type, is_dict_type, is_records_type, is_dataframe_type
-from gscraper.base.types import is_kwargs_allowed
+from gscraper.base.types import init_origin, is_list_type, is_dict_type, is_records_type, is_dataframe_type
+from gscraper.base.types import is_comparable, is_kwargs_allowed
 
 from gscraper.utils import isna, notna, empty, exists
 from gscraper.utils.cast import cast_str, cast_list, cast_tuple, cast_set
@@ -12,6 +12,7 @@ from gscraper.utils.cast import cast_str, cast_list, cast_tuple, cast_set
 from typing import Any, Callable, Dict, List, Set
 from typing import Iterable, Literal, Optional, Sequence, Tuple, Union
 from bs4 import BeautifulSoup, Tag
+from pandas.core.indexes.base import Index as PandasIndex
 import pandas as pd
 
 from collections import defaultdict
@@ -30,11 +31,25 @@ SOURCE_SEQUENCE_TYPE_MSG = "Required array of HTML source as input."
 LENGTH_MISMATCH_MSG = lambda left, right: f"Length of two input values are mismatched: [{left}, {right}]"
 
 
-arg_or = lambda *args: functools.reduce(lambda x,y: x|y, args)
-arg_and = lambda *args: functools.reduce(lambda x,y: x&y, args)
-union = lambda *arrays: functools.reduce(lambda x,y: x+y, arrays)
-inter = lambda *arrays: functools.reduce(lambda x,y: [e for e in x if e in y], arrays)
-diff = lambda *arrays: functools.reduce(lambda x,y: [e for e in x if e not in y], arrays)
+def arg_or(*args: bool) -> bool:
+    if not args: return False
+    else: return functools.reduce(lambda x,y: x|y, args)
+
+def arg_and(*args: bool) -> bool:
+    if not args: return False
+    else: return functools.reduce(lambda x,y: x&y, args)
+
+def union(*arrays) -> Any:
+    if not arrays: return None
+    else: return functools.reduce(lambda x,y: x+y, arrays)
+
+def inter(*arrays: Sequence) -> List:
+    if not arrays: return list()
+    else: return functools.reduce(lambda x,y: [e for e in x if e in y], arrays)
+
+def diff(*arrays: Sequence) -> List:
+    if not arrays: return list()
+    else: return functools.reduce(lambda x,y: [e for e in x if e not in y], arrays)
 
 
 def isna_plus(__object, strict=True) -> bool:
@@ -82,7 +97,7 @@ def to_array(__object, default=None, dropna=False, strict=False, unique=False) -
     else: return __s
 
 
-def to_dict(__object: MappingData, orient: Optional[Literal["dict","list","index"]]="dict", depth=0) -> Dict:
+def to_dict(__object: MappingData, orient: Optional[Literal["dict","list","index"]]="dict", depth=1) -> Dict:
     if isinstance(__object, Dict): return __object
     elif is_records(__object, empty=True): return chain_dict(__object, keep="first")
     elif isinstance(__object, pd.DataFrame):
@@ -90,7 +105,7 @@ def to_dict(__object: MappingData, orient: Optional[Literal["dict","list","index
     else: return dict()
 
 
-def to_records(__object: MappingData, depth=0) -> Records:
+def to_records(__object: MappingData, depth=1) -> Records:
     if is_records(__object, empty=True): return __object
     elif isinstance(__object, pd.DataFrame):
         return fillna_records(__object.to_dict("records"), value=None, strict=True, depth=depth)
@@ -98,11 +113,11 @@ def to_records(__object: MappingData, depth=0) -> Records:
     else: return list()
 
 
-def to_dataframe(__object: MappingData, index: Optional[Sequence]=None) -> pd.DataFrame:
+def to_dataframe(__object: MappingData, columns: Optional[IndexLabel]=None, index: Optional[Sequence]=None) -> pd.DataFrame:
     if isinstance(__object, pd.DataFrame): pass
-    elif is_records(__object, empty=True): __object = convert_dtypes(pd.DataFrame(align_records(__object)))
+    elif is_records(__object, empty=False): __object = convert_dtypes(pd.DataFrame(align_records(__object)))
     elif isinstance(__object, (dict,pd.Series)): __object = pd.DataFrame([__object])
-    else: return pd.DataFrame()
+    else: return pd.DataFrame(columns=cast_columns(columns))
     if (index is not None) and (safe_len(index, -1) == len(__object)): __object.index = index
     return __object
 
@@ -110,35 +125,31 @@ def to_dataframe(__object: MappingData, index: Optional[Sequence]=None) -> pd.Da
 def to_series(__object: Union[pd.Series,Sequence,Any], index: Optional[Sequence]=None) -> pd.Series:
     if isinstance(__object, pd.Series): pass
     elif is_array(__object): __object = pd.Series(__object)
-    else: __object = pd.Series([__object]*safe_len(index, 1))
+    else: __object = pd.Series([__object]*safe_len(index, 1), dtype="object")
     if safe_len(index, -1) == len(__object): __object.index = index
     return __object
 
 
-def convert_data(data: Data, return_type: Optional[TypeHint]=None, depth=0, **kwargs) -> Data:
+def convert_data(data: Data, return_type: Optional[TypeHint]=None, depth=1) -> Data:
     if not return_type: return data
     elif is_records_type(return_type): return to_records(data, depth=depth)
     elif is_dataframe_type(return_type): return to_dataframe(data)
     elif is_dict_type(return_type): return to_dict(data, depth=depth)
     elif is_list_type(return_type): return cast_list(data)
-    else: return data
-
-
-def multitype_allowed(func):
-    @functools.wraps(func)
-    def wrapper(data: Data, *args, return_type: Optional[TypeHint]=None, convert_first=False, **kwargs):
-        if not return_type: return func(data, *args, **kwargs)
-        if convert_first: data, return_type = convert_data(data, return_type), None
-        data = func(data, *args, **kwargs)
-        if not convert_first: data = convert_data(data, return_type)
-        return data
-    return wrapper
+    try: return data if data else init_origin(return_type)
+    except: return None
 
 
 ###################################################################
 ############################## Rename #############################
 ###################################################################
 
+def rename_value(__s: str, rename: RenameMap, to: Optional[Literal["key","value"]]="value",
+                if_null: Union[Literal["pass"],Any]="pass") -> str:
+    renameMap = flip_dict(rename) if to == "key" else rename
+    if renameMap and (__s in renameMap): return renameMap[__s]
+    else: return __s if if_null == "pass" else if_null
+
 def rename_dict(__m: Dict, rename: RenameMap) -> Dict:
     return {rename.get(__key,__key): __value for __key, __value in __m.items()}
 
@@ -148,9 +159,7 @@ def rename_records(__r: Records, rename: RenameMap) -> Records:
     return [rename_dict(__m, rename=rename) for __m in __r]
 
 
-@multitype_allowed
-def rename_data(data: MappingData, rename: RenameMap,
-                return_type: Optional[TypeHint]=None, convert_first=False, **kwargs) -> MappingData:
+def rename_data(data: MappingData, rename: RenameMap) -> MappingData:
     if not rename: return data
     elif isinstance(data, Dict): return rename_dict(data, rename)
     elif is_records(data): return rename_records(data, rename)
@@ -159,17 +168,6 @@ def rename_data(data: MappingData, rename: RenameMap,
     else: return rename.get(data,data)
 
 
-def multitype_rename(func):
-    @functools.wraps(func)
-    def wrapper(data: Data, *args, rename: RenameMap=dict(), rename_first=False, **kwargs):
-        if not rename: return func(data, *args, **kwargs)
-        if rename_first: data = rename_data(data, rename)
-        data = func(data, *args, **kwargs)
-        if not rename_first: data = rename_data(data, rename)
-        return data
-    return wrapper
-
-
 ###################################################################
 ############################### Get ###############################
 ###################################################################
@@ -186,8 +184,8 @@ def iloc(__s: IndexedSequence, indices: Index, default=None, if_null: Literal["d
     else: return [(__s[__i] if abs_idx(__i) < __length else default) for __i in __indices]
 
 
-def kloc(__m: Dict, keys: _KT, default=None, if_null: Literal["drop","pass"]="drop",
-        reorder=True, values_only=False, hier=False, key_alias: Sequence[_KT]=list()) -> Union[_VT,Dict]:
+def kloc(__m: Dict, keys: _KT, default=None, if_null: Literal["drop","pass"]="drop", reorder=True,
+        values_only=False, hier=False, key_alias: Sequence[_KT]=list()) -> Union[_VT,Dict]:
     if hier:
         return hloc(__m, keys, default, if_null=if_null, values_only=values_only, key_alias=key_alias)
     elif not is_array(keys):
@@ -213,7 +211,7 @@ def hloc(__m: Dict, keys: Sequence[_KT], default=None, if_null: Literal["drop","
     return list(__m.values()) if values_only else __m
 
 
-def vloc(__r: List[Dict], keys: _KT, default=None, if_null: Literal["drop","pass"]="drop", reorder=True,
+def vloc(__r: Records, keys: _KT, default=None, if_null: Literal["drop","pass"]="drop", reorder=True,
         values_only=False, hier=False, key_alias: Sequence[_KT]=list(), axis=0) -> Union[Records,List,Dict[_KT,List]]:
     if axis == 1:
         __m = {__key: vloc(__r, __key, default=default, if_null=if_null, hier=hier) for __key in cast_tuple(keys)}
@@ -228,14 +226,14 @@ def vloc(__r: List[Dict], keys: _KT, default=None, if_null: Literal["drop","pass
 
 def cloc(df: PandasData, columns: IndexLabel, default=None, if_null: Literal["drop","pass"]="drop",
         reorder=True, values_only=False) -> Union[PandasData,_VT]:
-    left, right = get_columns(df), cast_tuple(columns)
-    __inter = inter(right, left) if reorder else inter(left, right)
+    columns = cast_columns(columns)
+    __inter = inter(columns, get_columns(df)) if reorder else inter(get_columns(df), columns)
     if not __inter:
-        return init_df(df) if if_null == "drop" else df
+        return init_df(df, columns=list()) if if_null == "drop" else df
     elif not is_array(columns): df = df[columns]
     elif (if_null == "pass") and (len(columns) != len(__inter)):
-        if reorder: df = pd.concat([init_df(df),df])
-        else: df = pd.concat([init_df(df, columns=unique(*__inter,*columns)),df])
+        columns = columns if reorder else unique(*__inter, *columns)
+        df = pd.concat([init_df(df, columns=columns), df[__inter]])
     else: df = df[__inter]
     df = fillna_data(df, default)
     if isinstance(df, pd.Series): return df.tolist() if values_only else df
@@ -272,12 +270,9 @@ def get_value(data: ResponseData, field: Union[_KT,Index], default=None, **kwarg
     else: return default
 
 
-@multitype_allowed
-@multitype_rename
 def filter_data(data: ResponseData, fields: Optional[Union[_KT,Index]]=list(), default=None,
                 if_null: Literal["drop","pass"]="drop", reorder=True, values_only=False, hier=False,
-                key_alias: Sequence[_KT]=list(), return_type: Optional[TypeHint]=None, rename: RenameMap=dict(),
-                convert_first=False, rename_first=False, **kwargs) -> Data:
+                key_alias: Sequence[_KT]=list(), **kwargs) -> Data:
     context = dict(default=default, if_null=if_null, values_only=values_only)
     map_context = dict(hier=hier, key_alias=key_alias)
     if not fields: return data if if_null == "pass" else default
@@ -289,19 +284,6 @@ def filter_data(data: ResponseData, fields: Optional[Union[_KT,Index]]=list(), d
     else: return data
 
 
-def multitype_filter(func):
-    @functools.wraps(func)
-    def wrapper(data: ResponseData, *args, fields: Optional[Union[_KT,Index]]=list(), default=None,
-                if_null: Literal["drop","pass"]="pass", reorder=True, values_only=False, hier=False,
-                filter_first=False, **kwargs):
-        if not fields: return func(data, *args, **kwargs) if if_null != "drop" else list()
-        if filter_first: data = filter_data(data, fields, default, if_null, reorder)
-        data = func(data, *args, **kwargs)
-        if not filter_first: data = filter_data(data, fields, default, if_null, reorder, values_only, hier)
-        return data
-    return wrapper
-
-
 ###################################################################
 ############################### Set ###############################
 ###################################################################
@@ -329,10 +311,8 @@ def set_df(df: PandasData, __columns: Optional[IndexLabel]=list(), __values: Opt
     return df
 
 
-@multitype_allowed
 def set_data(data: MappingData, __keys: Optional[_KT]=list(), __values: Optional[_VT]=list(),
-            if_exists: Literal["replace","ignore"]="replace",
-            return_type: Optional[TypeHint]=None, convert_first=False, **context) -> Data:
+            if_exists: Literal["replace","ignore"]="replace", **context) -> Data:
     if is_records(data): return set_records(data, __keys, __values, if_exists=if_exists, **context)
     elif isinstance(data, PANDAS_DATA): return set_df(data, __keys, __values, if_exists=if_exists, **context)
     elif isinstance(data, Dict): return set_dict(data, __keys, __values, if_exists=if_exists, inplace=False, **context)
@@ -388,12 +368,16 @@ def exists_one(*args, strict=False) -> Any:
     if args: return args[-1]
 
 
-def notna_dict(__m: Optional[Dict]=dict(), strict=True, **context) -> Dict:
-    return {__k: __v for __k, __v in dict(__m, **context).items() if notna(__v, strict=strict)}
+def notna_dict(__m: Optional[Dict]=dict(), null_if: Dict=dict(), strict=True, **context) -> Dict:
+    if not null_if:
+        return {__k: __v for __k, __v in dict(__m, **context).items() if notna(__v, strict=strict)}
+    else: return {__k: __v for __k, __v in dict(__m, **context).items() if notna(__v, strict=strict) and (__v != null_if.get(__k))}
 
 
-def exists_dict(__m: Optional[Dict]=dict(), strict=False, **context) -> Dict:
-    return {__k: __v for __k, __v in dict(__m, **context).items() if exists(__v, strict=strict)}
+def exists_dict(__m: Optional[Dict]=dict(), null_if: Dict=dict(), strict=False, **context) -> Dict:
+    if not null_if:
+        return {__k: __v for __k, __v in dict(__m, **context).items() if exists(__v, strict=strict)}
+    else: return {__k: __v for __k, __v in dict(__m, **context).items() if exists(__v, strict=strict) and (__v != null_if.get(__k))}
 
 
 def df_empty(df: pd.DataFrame, drop_na=False, how: Literal["any","all"]="all") -> bool:
@@ -611,13 +595,7 @@ def concat_df(__object: Sequence[pd.DataFrame], axis=0, keep: Literal["fist","la
     return pd.concat(__object, axis=axis) if __object else pd.DataFrame()
 
 
-@multitype_allowed
-@multitype_rename
-@multitype_filter
-def chain_exists(data: Data, data_type: Optional[TypeHint]=None, keep: Literal["fist","last"]="first",
-                fields: Optional[Union[_KT,Index]]=list(), default=None, if_null: Literal["drop","pass"]="drop",
-                reorder=True, values_only=False, hier=False, return_type: Optional[TypeHint]=None,
-                rename: RenameMap=dict(), convert_first=False, rename_first=False, filter_first=False, **kwargs) -> Data:
+def chain_exists(data: Data, data_type: Optional[TypeHint]=None, keep: Literal["fist","last"]="first") -> Data:
     if is_dfarray(data): return concat_df([df for df in data if df_exists(df)])
     elif is_2darray(data): return list(chain.from_iterable([__s for __s in data if is_array(__s)]))
     elif data_type and is_dict_type(data_type) and is_records(data, how="any"):
@@ -711,7 +689,7 @@ def apply_dict(__m: Dict, __keys: Optional[_KT]=list(), __applyFunc: Optional[Ap
     return __m
 
 
-def apply_records(__r: List[Dict], __keys: Optional[_KT]=list(), __applyFunc: Optional[ApplyFunction]=list(),
+def apply_records(__r: Records, __keys: Optional[_KT]=list(), __applyFunc: Optional[ApplyFunction]=list(),
                     apply: Optional[ApplyFunction]=None, all_keys=False, scope: Literal["keys","dict"]="keys",
                     __default=None, **context) -> Records:
     if not __r: return __r
@@ -747,14 +725,8 @@ def safe_apply_df(__object: PandasData, __applyFunc: ApplyFunction, default: Uni
     return fillna_data(__object, default)
 
 
-@multitype_allowed
-@multitype_rename
-@multitype_filter
 def apply_data(data: Data, __keys: Optional[Union[_KT,Index]]=list(), __applyFunc: Optional[ApplyFunction]=list(),
-                apply: Optional[ApplyFunction]=None, all_keys=False, fields: Optional[Union[_KT,Index]]=list(),
-                default=None, if_null: Literal["drop","pass"]="drop", reorder=True, values_only=False, hier=False,
-                return_type: Optional[TypeHint]=None, rename: RenameMap=dict(),
-                convert_first=False, rename_first=False, filter_first=False, **context) -> Data:
+                apply: Optional[ApplyFunction]=None, all_keys=False, default=None, **context) -> Data:
     if isinstance(data, Dict): return apply_dict(data, __keys, __applyFunc, apply=apply, all_keys=all_keys, **context)
     if is_records(data): return apply_records(data, __keys, __applyFunc, apply=apply, all_keys=all_keys, **context)
     elif isinstance(data, PANDAS_DATA): return apply_df(data, __keys, __applyFunc, apply=apply, all_cols=all_keys, **context)
@@ -799,7 +771,7 @@ def match_dict(__m: Dict, __keys: Optional[_KT]=list(), __matchFunc: Optional[Ma
     else: return all(matches.values()) if how == "all" else matches
 
 
-def match_records(__r: List[Dict], __keys: Optional[_KT]=list(), __matchFunc: Optional[MatchFunction]=list(),
+def match_records(__r: Records, __keys: Optional[_KT]=list(), __matchFunc: Optional[MatchFunction]=list(),
                 match: Optional[MatchFunction]=None, all_keys=False, how: Literal["filter","all","indexer"]="filter",
                 scope: Literal["keys","dict"]="keys", __default=None, **context) -> Union[Records,_BOOL]:
     if not __r: return False if how == "all" else list()
@@ -821,14 +793,9 @@ def match_df(df: PandasData, __columns: Optional[IndexLabel]=list(), __matchFunc
     else: return matches.all(axis=0) if how == "all" else matches
 
 
-@multitype_allowed
-@multitype_rename
-@multitype_filter
 def match_data(data: Data, __keys: Optional[Union[_KT,Index]]=list(), __matchFunc: Optional[MatchFunction]=list(),
                 match: Optional[MatchFunction]=None, all_keys=False, how: Literal["filter","all","indexer"]="filter",
-                fields: Optional[Union[_KT,Index]]=list(), default=None, if_null: Literal["drop","pass"]="drop",
-                reorder=True, values_only=False, hier=False, return_type: Optional[TypeHint]=None, rename: RenameMap=dict(),
-                convert_first=False, rename_first=False, filter_first=False, **context) -> Data:
+                fields: Optional[Union[_KT,Index]]=list(), default=None, **context) -> Data:
     if isinstance(data, Dict): return match_dict(data, __keys, __matchFunc, match=match, all_keys=all_keys, how=how, **context)
     if is_records(data): return match_records(data, __keys, __matchFunc, match=match, all_keys=all_keys, how=how, **context)
     elif isinstance(data, PANDAS_DATA): return match_df(data, __keys, __matchFunc, match=match, all_cols=all_keys, how=how, **context)
@@ -911,9 +878,9 @@ def _from_selector_path(__path: _KT, key=str(), text=False) -> Tuple[_KT,str,str
         elif __path.startswith('.'): return (str(), "class", "attr")
         elif __path.startswith('#'): return (str(), "id", "attr")
         else: return (__path, str(), ("text" if text else "source"))
-    elif is_array(__path):
+    elif is_array(__path) and __path:
         selector, key, by = _from_selector_path(__path[-1])
-        return ((__path+[selector] if selector else __path), key, by)
+        return ((__path[:-1]+[selector] if selector else __path[:-1]), key, by)
     else: return (str(), str(), "source")
 
 
@@ -1040,13 +1007,8 @@ def between_df(df: pd.DataFrame, __columns: Optional[IndexLabel]=list(), __range
     return df
 
 
-@multitype_allowed
-@multitype_rename
-@multitype_filter
-def between_data(data: MappingData, inclusive: Literal["both","neither","left","right"]="both", null=False,
-                fields: Optional[Union[_KT,Index]]=list(), default=None, if_null: Literal["drop","pass"]="drop",
-                reorder=True, values_only=False, hier=False, return_type: Optional[TypeHint]=None, rename: RenameMap=dict(),
-                convert_first=False, rename_first=False, filter_first=False, **context) -> MappingData:
+def between_data(data: MappingData, inclusive: Literal["both","neither","left","right"]="both",
+                null=False, **context) -> MappingData:
     if is_records(data): return between_records(data, inclusive=inclusive, null=null, **context)
     elif isinstance(data, pd.DataFrame): return between_df(data, inclusive=inclusive, null=null, **context)
     elif isinstance(data, Dict): return data if between_dict(data, inclusive=inclusive, null=null, **context) else dict()
@@ -1065,6 +1027,11 @@ def drop_dict(__m: Dict, keys: _KT, inplace=False) -> Dict:
     if not inplace: return __m
 
 
+def split_dict(__m: Dict, keys: _KT) -> Tuple[Dict,Dict]:
+    keys = cast_tuple(keys)
+    return kloc(__m, keys, if_null="drop"), drop_dict(__m, keys, inplace=False)
+
+
 def drop_records(__r: Records, keys: _KT) -> Records:
     return [drop_dict(__m, keys, inplace=False) for __m in __r]
 
@@ -1077,9 +1044,7 @@ def drop_df(df: PandasData, columns: IndexLabel) -> PandasData:
     else: return df
 
 
-@multitype_allowed
-def drop_data(data: MappingData, __keys: Optional[_KT]=list(),
-            return_type: Optional[TypeHint]=None, convert_first=False, **kwargs) -> Data:
+def drop_data(data: MappingData, __keys: Optional[_KT]=list()) -> Data:
     if is_records(data): return drop_records(data, __keys)
     elif isinstance(data, PANDAS_DATA): return drop_df(data, __keys)
     elif isinstance(data, Dict): return drop_dict(data, __keys, inplace=False)
@@ -1134,15 +1099,13 @@ def fill_array(__s: Sequence, count: int, value=None) -> List:
 
 def fillna_array(__s: Sequence, value=None, strict=True, depth=1) -> List:
     if depth < 1: return __s
-    fillna = lambda __value: fillna_data(
-        __value, value, strict=strict, depth=depth-1) if depth > 1 else __value
+    fillna = lambda __value: fillna_data(__value, value, strict=strict, depth=depth-1) if depth > 1 else __value
     return [(fillna(__e) if notna(__e, strict=strict) else value) for __e in __s]
 
 
 def fillna_dict(__m: Dict, value=None, strict=True, depth=1) -> Dict:
     if depth < 1: return __m
-    fillna = lambda __value: fillna_data(
-        __value, value, strict=strict, depth=depth-1) if depth > 1 else __value
+    fillna = lambda __value: fillna_data(__value, value, strict=strict, depth=depth-1) if depth > 1 else __value
     return {__key: (fillna(__value) if notna(__value, strict=strict) else value) for __key, __value in __m.items()}
 
 
@@ -1175,10 +1138,8 @@ def fillna_series(__object: pd.Series, value: Union[pd.Series,Sequence,Any]=None
     else: return __object.fillna(value)
 
 
-@multitype_allowed
 def fillna_data(__object: Union[PandasData,Records], value: Union[PandasData,Sequence,Any]=None,
-                strict=True, depth=1, return_type: Optional[TypeHint]=None, convert_first=False,
-                **kwargs) -> Union[PandasData,Records]:
+                strict=True, depth=1) -> Union[PandasData,Records]:
     if isna_plus(value) and isinstance(__object, PANDAS_DATA): return __object
     elif isinstance(__object, pd.DataFrame): return fillna_df(__object, value)
     elif isinstance(__object, pd.Series): return fillna_series(__object, value)
@@ -1208,13 +1169,7 @@ def sort_records(__r: Records, by: _KT, ascending=True) -> Records:
     return sorted(__r, key=lambda x: kloc(x, cast_tuple(by), values_only=True), reverse=(not ascending))
 
 
-@multitype_allowed
-@multitype_rename
-@multitype_filter
-def sort_values(data: TabularData, by: _KT, ascending: _BOOL=True, fields: Optional[Union[_KT,Index]]=list(),
-                default=None, if_null: Literal["drop","pass"]="drop", reorder=True, values_only=False, hier=False,
-                return_type: Optional[TypeHint]=None, rename: RenameMap=dict(), convert_first=False, rename_first=False,
-                filter_first=False, **kwargs) -> Data:
+def sort_values(data: TabularData, by: _KT, ascending: _BOOL=True) -> Data:
     asc = ascending if isinstance(ascending, bool) else bool(get_scala(ascending))
     if is_records(data): return sort_records(data, by=by, ascending=asc)
     elif isinstance(data, pd.DataFrame): return data.sort_values(by, ascending=ascending)
@@ -1324,15 +1279,25 @@ def unit_array(__s: Sequence, unit=1) -> List[Sequence]:
 ############################ DataFrame ############################
 ###################################################################
 
-def init_df(df: PandasData, columns: IndexLabel=list()) -> PandasData:
+def init_df(df: Optional[PandasData]=None, columns: Optional[IndexLabel]=None) -> PandasData:
     if isinstance(df, pd.DataFrame):
-        return pd.DataFrame(columns=(columns if columns else df.columns))
+        return pd.DataFrame(columns=(columns if is_columns(columns) else df.columns))
     elif isinstance(df, pd.Series):
         return pd.Series(index=(columns if columns else df.index), dtype=df.dtypes)
-    else: return
+    else: return pd.DataFrame(columns=cast_columns(columns))
 
 
-def get_columns(df: PandasData) -> IndexLabel:
+def is_columns(columns: IndexLabel) -> bool:
+    return is_array(columns) or isinstance(columns, PandasIndex)
+
+
+def cast_columns(__object, unique=False) -> IndexLabel:
+    if isinstance(__object, PandasIndex):
+        return (__object.unique() if unique else __object).tolist()
+    else: return cast_list(__object)
+
+
+def get_columns(df: PandasData) -> PandasIndex:
     if isinstance(df, pd.DataFrame): return df.columns
     elif isinstance(df, pd.Series): return df.index
     else: return list()
